"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Loader = exports.TTYSync = void 0;
const node_events_1 = require("node:events");
async function sleep(duration) {
    return new Promise((resolve) => setTimeout(resolve, duration));
}
class TTYSync {
    static moveCursorSync(dx, dy) {
        return new Promise((resolve) => process.stdout.moveCursor(dx, dy, resolve));
    }
    static cursorToSync(x, y) {
        return new Promise((resolve) => process.stdout.cursorTo(x, y, resolve));
    }
    static clearLineSync(direction) {
        return new Promise((resolve) => process.stdout.clearLine(1, resolve));
    }
    // clear number of rows above starting from the current row
    static clearAboveUntilSync(numberOfRowsToDelete) {
        return new Promise(async (resolve) => {
            // assume that cursor might not be at the beginning of line
            await this.cursorToSync(0);
            for (let i = 0; i < numberOfRowsToDelete; i++) {
                await this.clearLineSync(1);
                await this.moveCursorSync(0, -1);
            }
            // cursor will always end up on the last deleted row
            await this.moveCursorSync(0, 1);
            resolve();
        });
    }
}
exports.TTYSync = TTYSync;
class Loader {
    constructor(text = "Loading") {
        this.text = text;
        this.shouldRun = false;
        this.eventEmitter = new node_events_1.EventEmitter();
    }
    setText(text) {
        this.text = text;
    }
    async start() {
        this.shouldRun = true;
        console.log(this.text);
        await TTYSync.moveCursorSync(this.text.length, -1);
        this.eventEmitter.on("stop", async () => {
            this.shouldRun = false;
        });
        for (let i = 0; this.shouldRun == true; i++) {
            console.log(".".repeat((i % 3) + 1));
            await sleep(300);
            await TTYSync.moveCursorSync(this.text.length, -1);
            await TTYSync.clearLineSync(1);
        }
        await TTYSync.cursorToSync(0);
        await TTYSync.clearLineSync(1);
        this.eventEmitter.emit("animation finish");
    }
    async stop(callback) {
        this.eventEmitter.emit("stop");
        await (() => {
            return new Promise((resolve) => this.eventEmitter.once("animation finish", () => resolve()));
        })();
        if (callback)
            callback();
    }
    startAsyncTask(action) {
        return new Promise(async (resolve, reject) => {
            this.start();
            try {
                const data = await action();
                await this.stop(() => resolve(data));
            }
            catch (e) {
                await this.stop();
                reject(e);
                // throw e;
            }
        });
    }
}
exports.Loader = Loader;
